/*
 * Project name
     Parking Statistic Demo
 * Copyright
     (c) mikroElektronika, 2011.
 * Revision History
     20110527:
       - initial release (SZ);
 * Description
       Let's make 6 project is actually a simple counter of free parking spots left at the parking place.
       When car enters the parking place the number of free parking spaces decrements. Similar, when a car leaves a parking
       space the number of parking spaces available increments.
       The initial value of free parking spaces can be set in three ways. First in code, also can be stored
       in EEPROM and you can enter it via Keypad 4x4.
                                  ac:Lets_make_6
 * Test configuration:
     MCU:             PIC18F887
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf

     Dev.Board:       PICReady1 Board
                      http://www.mikroe.com/eng/products/view/305/pic-ready1-board/
                      ac:PIC-Ready1_Board

     Oscillator:      HS, 8.0000 MHz

     Ext. Modules:    Distance_Meter2 Boards  (ADC input at PORTA, digital at PORTB and PORTH)
                      http://www.mikroe.com/eng/products/view/438/distance-meter-2-board/
                      ac:Distance_Meter2_Board

                      Serial 7-seg Display Board at PORTC
                      http://www.mikroe.com/eng/products/view/162/serial-7-seg-display-board/
                      ac:Serial_7-seg_Display_Board
                      
                      Keypad 4x4 Board at PORTD
                      http://www.mikroe.com/eng/products/view/135/keypad-4x4-board/
                      ac:Keypad_4x4_Board

                      EasyPULL Board at PORTD
                      http://www.mikroe.com/eng/products/view/574/easypull-board/
                      ac:EasyPULL_Board

                      EasyTest Board at PORTA
                      http://www.mikroe.com/eng/products/view/183/easytest-board/
                      ac:EasyTest_Board

     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
    - Pull down Keypad4x4 using the EasyPULL Board.
    - Distance Meter2 Boards settings:
              Entrance:   SW1.1, SW1.5 and SW1.7 - ON;
              Exit:       SW1.2, SW1.5 and SW1.7 - ON;
      Note that both upper flat cables of DistanceMeter2 Boards are connected to PORTA using the EasyTest Board!
    - All calculation in this example are based for 8MHz crystal!
*/


#include <built_in.h>

// Keypad module connections
char  keypadPort at PORTD;
// End Keypad module connections


// Soft PWM
sbit PWMA at RB2_bit;
sbit PWMB at RB1_bit;
sbit PWMA2 at RE2_bit;
sbit PWMB2 at RE1_bit;

sbit PWMA_Direction at TRISB2_bit;
sbit PWMB_Direction at TRISB1_bit;
sbit PWMA2_Direction at TRISE2_bit;
sbit PWMB2_Direction at TRISE1_bit;
//

// Serial 7-seg Display connections
sbit MAX7219_Chip_Select at RC0_bit;
sbit MAX7219_Chip_Select_Direction at TRISC0_bit;
//

unsigned int Parking_spots_remained = 255;
float Distance;
char distance_txt[15];
char entrance, entrance_old = 0;
char exit, exit_old = 0;


// Init MAX7219
void MAX7219_Init() {
  MAX7219_Chip_Select = 0;       // Select MAX7219
  SPI1_write(0x09);              // BCD mode for digit decoding
  SPI1_write(0xFF);
  MAX7219_Chip_Select = 1;       // Deselect MAX7219

  MAX7219_Chip_Select = 0;
  SPI1_write(0x0A);
  SPI1_write(0x0F);              // Segment luminosity intensity
  MAX7219_Chip_Select = 1;

  MAX7219_Chip_Select = 0;
  SPI1_write(0x0B);
  SPI1_write(0x07);              // Display refresh
  MAX7219_Chip_Select = 1;

  MAX7219_Chip_Select = 0;
  SPI1_write(0x0C);
  SPI1_write(0x01);              // Turn on the display
  MAX7219_Chip_Select = 1;

  MAX7219_Chip_Select = 0;
  SPI1_write(0x00);
  SPI1_write(0xFF);              // No test
  MAX7219_Chip_Select = 1;
}

// Display number at 7SEGs
void Display_7Seg(unsigned int num) {

  MAX7219_Chip_Select = 0;
  SPI1_write(4);                 // Send Thousands digit
  SPI1_write(num/1000);
  MAX7219_Chip_Select = 1;

  MAX7219_Chip_Select = 0;
  SPI1_write(3);                 // Send Hundreds digit
  SPI1_write((num/100)%10);
  MAX7219_Chip_Select = 1;

  MAX7219_Chip_Select = 0;
  SPI1_write(2);                 // Send Tens digit
  SPI1_write((num/10)%10);
  MAX7219_Chip_Select = 1;
  
  MAX7219_Chip_Select = 0;
  SPI1_write(1);                 // Send Ones digit
  SPI1_write(num%10);
  MAX7219_Chip_Select = 1;
}

// Blink_7Seg display 3 times
void Blink_7Seg() {
char i;
for (i = 0; i < 3; i++){
      MAX7219_Chip_Select = 0;
      SPI1_write(0x0A);
      SPI1_write(0x00);          // Segment luminosity intensity
      MAX7219_Chip_Select = 1;
      Delay_ms(250);
      MAX7219_Chip_Select = 0;
      SPI1_write(0x0A);
      SPI1_write(0x0F);          // Segment luminosity intensity
      MAX7219_Chip_Select = 1;
      Delay_ms(250);
  }
}

// Generation of 10 (Soft PWM) pulses - Entrance
void SoftPWM_Entrance(){
char i;

  PWMA_Direction = 0;

  PWMA = 0;
  PWMB = 0;

 for (i = 0; i < 10; i++){
  PWMA = 1;
  PWMB = 0;
  Delay_us(10);
  PWMA = 0;
  PWMB = 1;
  Delay_us(10);
 }

  PWMA_Direction = 1;

  PWMA = 0;
  PWMB = 0;

}

// Generation of 10 (Soft PWM) pulses - Exit
void SoftPWM_Exit(){
char i;
  PWMA2_Direction = 0;

  PWMA2 = 0;
  PWMB2 = 0;

 for (i = 0; i < 10; i++){
  PWMA2 = 1;
  PWMB2 = 0;
  Delay_us(10);
  PWMA2 = 0;
  PWMB2 = 1;
  Delay_us(10);
 }

  PWMA2_Direction = 1;

  PWMA2 = 0;
  PWMB2 = 0;

}


char Detect_Obstacle(void (*Soft_PWM)(void), char adc_channel, float* dist) {
// In dry air at 20 C (68 F), the speed of sound is 343.2 m/s
// If obstacle is 2m far from the MB1100 sensor then sound will have to travel 4m
// (2m from the MB1100 and towards the obstacle, then reflect from it and travel back to the MB1100 another 2m).
// Duration of that 4m trip is t = s/v = 4m / 343.2m/s = 11.765ms
// Note that the speed of sound depends of many factors like air temperature, humidity, pressure etc.
// For our project we'll ignore these influences because we don't want to find out how far the obstacle is
// but only to detects its presence.
// Timer clock is 250KHz which gives the time-measuring resolution of 4us.
// Therefore TMR1H:TMR1L = 2941 is the maximal obsticle detection duration (11.765ms / 4us = 2941.25).

 unsigned int time, time2MAX = 0, adc_t, adc_t_plus_delta_t, adc_tmp, adc_tmp_old = 0;

  TMR1ON_bit = 0;                               // Stop the timer
  TMR1L = 0x00;                                 // Initial value of TMR1 register
  TMR1H = 0x00;
  TMR1ON_bit = 1;                               // Start Timer1

  Soft_PWM();                                   // Apply 10 Soft PWM impulses

  Delay_us(100);                                // Piezzo settling down time


// We will detect the first echo pulse. This one must be ignored because it is the noise
// generated by the ultrasonic transmitter.

  adc_t = ADC_Get_Sample(adc_channel);          // Get first two samples
  Delay_us(1);
  adc_t_plus_delta_t = ADC_Get_Sample(adc_channel);

// Detect the max adc value of the noise impulse
  while (adc_t_plus_delta_t > adc_t) {
      Delay_us(10);
      adc_t = ADC_Get_Sample(adc_channel);
      Delay_us(1);
      adc_t_plus_delta_t = ADC_Get_Sample(adc_channel);
  }
//

/*  time1MAX = TMR1H << 8;
  time1MAX |= TMR1L;
  MAX1_Adc = adc_t;*/

// Detect the lowest adc value of the noise impulse
  adc_t = ADC_Get_Sample(adc_channel);                    // Get first two samples
  Delay_us(1);
  adc_t_plus_delta_t = ADC_Get_Sample(adc_channel);

  while (adc_t_plus_delta_t < adc_t) {
      Delay_us(10);
      adc_t = ADC_Get_Sample(adc_channel);
      Delay_us(1);
      adc_t_plus_delta_t = ADC_Get_Sample(adc_channel);
  }

/*  time_MIN = TMR1H << 8;
  time_MIN |= TMR1L;
  MIN_Adc = adc_t_plus_delta_t;*/   // You could use the MIN_Adc for ADC Treshold calculation


//

// Now, we will search for the the second max adc value (of the impulse reflected from the obstacle)
// We'll do so until the end of detection period (11.765ms)

  do {
      time = TMR1H << 8;
      time |= TMR1L;

      adc_tmp = ADC_Get_Sample(adc_channel);              // Get sample

      if (adc_tmp > adc_tmp_old) {                        // If new value is larger than old one,
            adc_tmp_old = adc_tmp;                        // memorize this value,
            time2MAX =  time;                             // time also
//            if (adc_tmp >= ADC_Treshold) time2MAX =  time;           // If using the ADC Treshold value

         }
      Delay_us(50);

  } while(time <= 2941);

 TMR1ON_bit = 0;


if (time2MAX){
//  distance[um] =  343.2m/s * time2MAX*4us/2 = 686.4 time2MAX um

  *dist = 686.4 * time2MAX/1000;                // Distance in mm
  if (*dist <= 1000) return 1; else return 0;   // If measured distance is less than 1m obstacle is detected, otherwise not
 }

 else return 0;

}


char Check_Entrance(){ // Check if there is a car entering the parking space
char j;
// We'll perform 16 detections in a row. If the obstacle is detected at least 8 times then the
// obstacle is present for sure
entrance = 0;
 for (j = 0; j < 16; j++) entrance += Detect_Obstacle(SoftPWM_Entrance, 0, &Distance);
  if (entrance > 8) entrance = 1; else entrance = 0;
//

  if(entrance == 1 ) {                // If obstacle is detected then,
       if(entrance_old == 1) {        // check if it was detected also in the previous check
           entrance_old = entrance;   // If it was we'll ignore it
           return 0;

       }else{
           entrance_old = entrance;   // else obstacle is really detected!
           return 1;
       }
    }
    else
    {
      entrance_old = entrance;
      return 0;
    }
}


char Check_Exit(){     // Check if there is a car leaving the parking space
char j;
// We'll perform 16 detections in a row. If the obstacle is detected at least 8 times then the
// obstacle is present for sure
 exit = 0;
 for (j = 0; j < 16; j++) exit += Detect_Obstacle(SoftPWM_Exit, 1, &Distance);
  if (exit > 8) exit = 1; else exit = 0;
//

  if(exit == 1 ) {                    // If obstacle is detected then,
       if(exit_old == 1) {            // check if it was detected also in the previous check
           exit_old = exit;           // If it was we'll ignore it,
           return 0;

       }else{
           exit_old = exit;           // else obstacle is really detected!
           return 1;
       }
    }
    else
    {
      exit_old = exit;
      return 0;
    }

}


void Read_Keypad(){
char digit_place = 4, kp;

  Parking_spots_remained = 0;
  Display_7Seg(Parking_spots_remained);

//  Enter decimal digits (from left to right. When you're done press '*'
  while(1){

   kp = 0;                            // Reset key code variable
    // Wait for key to be pressed and released
   do
      kp = Keypad_Key_Click();        // Store key code in kp variable
   while (!kp);

      switch (kp) {
            case  1: kp = 1; break;   // 1
            case  2: kp = 2; break;   // 2
            case  3: kp = 3; break;   // 3
            case  5: kp = 4; break;   // 4
            case  6: kp = 5; break;   // 5
            case  7: kp = 6; break;   // 6
            case  9: kp = 7; break;   // 7
            case 10: kp = 8; break;   // 8
            case 11: kp = 9; break;   // 9
            case 13: kp = 255; break; // *
            case 14: kp = 0; break;   // 0

            default: kp = 100;
      }

    if (kp == 255) {
             Blink_7Seg();
             break;
    }

     if(kp != 100){
              switch (digit_place--) {
                case  1: {Parking_spots_remained += kp; digit_place = 4; break;}
                case  2: Parking_spots_remained += kp*10;  break;
                case  3: Parking_spots_remained += kp*100;  break;
                case  4: Parking_spots_remained = kp*1000;  break;

          }
       Display_7Seg(Parking_spots_remained);
     }

  }
}





void main() {
  ANSEL = 0x03;                                     // Set RA0 and RA1 pin as Analog Inputs
  ANSELH = 0;                                       // and other as digital I/Os
  C1ON_bit = 0;                                     // Disable comparators
  C2ON_bit = 0;
  TRISA0_bit = 1;                                   // RA0 as input
  TRISA1_bit = 1;                                   // RA1 as input

  PWMA = 0;                                         // Soft PWM pins init
  PWMB = 0;
  PWMA_Direction = 0;
  PWMB_Direction = 0;

  PWMA2 = 0;
  PWMB2 = 0;
  PWMA2_Direction = 0;
  PWMB2_Direction = 0;

  MAX7219_Chip_Select_Direction = 0;                // Set Chip select pin as output
  SPI1_init();                                      // Init SPI module
  MAX7219_Init();                                   // Init MAX7219

  T1CON.B5 = 1;                                     // Timer1 prescaler 1:8
  T1CON.B4 = 1;

  ADC_Init();                                       // Init ADC
  UART1_Init(9600);
  Keypad_Init();                                    // Initialize Keypad
  Delay_ms(500);

// Uncomment the code below if the initial number of parking spots remained is stored in EEPROM
/*  Lo(Parking_spots_remained) = EEPROM_Read(0);
  Hi(Parking_spots_remained) = EEPROM_Read(1);*/
//

// Enter the initial number of parking spots remained via 4x4 Keypad
 Read_Keypad();
//

 Delay_ms(2000);

 entrance_old = 0;
 exit_old = 0;
 
 while(1){
  Display_7Seg(Parking_spots_remained);             // Display the number of remaining parking spots at 7Seg display

  if (Check_Entrance()) Parking_spots_remained--;   // Check if the vehicle entered the parking space
                                                    // and if so decrement the number of remaining parking spots
  if (Check_Exit()) Parking_spots_remained++;       // Check if the vehicle exited the parking space
                                                    // and if so increment the number of remaining parking spots
 // For debug purposes:
/*  FloatToStr(Distance, distance_txt);
  strcat(distance_txt, "mm");
  UART1_Write_Text(distance_txt);
  UART1_Write_Text("\r\n");
  Delay_ms(500);*/
//

 }

}